@@ -34,6 +34,7 @@
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
+#include <inttypes.h>
 #include <dlfcn.h>
 #include <ctype.h>
 #include <stdarg.h>
@@ -203,6 +204,7 @@ static int inverse_for_options[NUMBER_OF_OPT] =
 /* 2 */ ARPT_INV_SRCDEVADDR,
 /* 3 */ ARPT_INV_TGTDEVADDR,
 /* -l */ ARPT_INV_ARPHLN,
+/* 8 */ 0,
 /* 4 */ ARPT_INV_ARPOP,
 /* 5 */ ARPT_INV_ARPHRD,
 /* 6 */ ARPT_INV_ARPPRO,
@@ -874,7 +876,7 @@ parse_target(const char *targetname)
 
 	if (strlen(targetname)+1 > sizeof(arpt_chainlabel))
 		exit_error(PARAMETER_PROBLEM,
-			   "Invalid target name `%s' (%i chars max)",
+			   "Invalid target name `%s' (%zu chars max)",
 			   targetname, sizeof(arpt_chainlabel)-1);
 
 	for (ptr = targetname; *ptr; ptr++)
@@ -1062,7 +1064,7 @@ register_match(struct arptables_match *me)
 	}
 
 	if (me->size != ARPT_ALIGN(me->size)) {
-		fprintf(stderr, "%s: match `%s' has invalid size %u.\n",
+		fprintf(stderr, "%s: match `%s' has invalid size %zu.\n",
 			program_name, me->name, me->size);
 		exit(1);
 	}
@@ -1092,7 +1094,7 @@ register_target(struct arptables_target *me)
 	}
 
 	if (me->size != ARPT_ALIGN(me->size)) {
-		fprintf(stderr, "%s: target `%s' has invalid size %u.\n",
+		fprintf(stderr, "%s: target `%s' has invalid size %zu.\n",
 			program_name, me->name, me->size);
 		exit(1);
 	}
@@ -1116,17 +1118,17 @@ print_num(u_int64_t number, unsigned int format)
 					number = (number + 500) / 1000;
 					if (number > 9999) {
 						number = (number + 500) / 1000;
-						printf(FMT("%4lluT ","%lluT "), number);
+						printf(FMT("%4"PRIu64"T ","%"PRIu64"T "), number);
 					}
-					else printf(FMT("%4lluG ","%lluG "), number);
+					else printf(FMT("%4"PRIu64"G ","%"PRIu64"G "), number);
 				}
-				else printf(FMT("%4lluM ","%lluM "), number);
+				else printf(FMT("%4"PRIu64"M ","%"PRIu64"M "), number);
 			} else
-				printf(FMT("%4lluK ","%lluK "), number);
+				printf(FMT("%4"PRIu64"K ","%"PRIu64"K "), number);
 		} else
-			printf(FMT("%5llu ","%llu "), number);
+			printf(FMT("%5"PRIu64" ","%"PRIu64" "), number);
 	} else
-		printf(FMT("%8llu ","%llu "), number);
+		printf(FMT("%8"PRIu64" ","%"PRIu64" "), number);
 }
 
 
@@ -1214,9 +1216,10 @@ print_firewall(const struct arpt_entry *fw,
 {
 	struct arptables_target *target = NULL;
 	const struct arpt_entry_target *t;
-	u_int8_t flags;
 	char buf[BUFSIZ];
 	int i;
+	char iface[IFNAMSIZ+2];
+	int print_iface = 0;
 
 	if (!arptc_is_chain(targname, handle))
 		target = find_target(targname, TRY_LOAD);
@@ -1224,7 +1227,6 @@ print_firewall(const struct arpt_entry *fw,
 		target = find_target(ARPT_STANDARD_TARGET, LOAD_MUST_SUCCEED);
 
 	t = arpt_get_target((struct arpt_entry *)fw);
-	flags = fw->arp.flags;
 
 	if (format & FMT_LINENUMBERS)
 		printf("%u ", num+1);
@@ -1232,35 +1234,34 @@ print_firewall(const struct arpt_entry *fw,
 	if (!(format & FMT_NOTARGET) && targname[0] != '\0')
 		printf("-j %s ", targname);
 
-	if (format & FMT_VIA) {
-		char iface[IFNAMSIZ+2];
+	iface[0] = '\0';
 
-		if (fw->arp.invflags & ARPT_INV_VIA_IN) {
-			iface[0] = '!';
-			iface[1] = '\0';
-		}
-		else iface[0] = '\0';
-
-		if (fw->arp.iniface[0] != '\0') {
-			strcat(iface, fw->arp.iniface);
-		}
-		else if (format & FMT_NUMERIC) strcat(iface, "*");
-		else strcat(iface, "any");
-		printf("-i %s ", iface);
-
-		if (fw->arp.invflags & ARPT_INV_VIA_OUT) {
-			iface[0] = '!';
-			iface[1] = '\0';
-		}
-		else iface[0] = '\0';
-
-		if (fw->arp.outiface[0] != '\0') {
-			strcat(iface, fw->arp.outiface);
-		}
-		else if (format & FMT_NUMERIC) strcat(iface, "*");
-		else strcat(iface, "any");
-		printf("-o %s ", iface);
+	if (fw->arp.iniface[0] != '\0') {
+		strcat(iface, fw->arp.iniface);
+		print_iface = 1;
 	}
+	else if (format & FMT_VIA) {
+		print_iface = 1;
+		if (format & FMT_NUMERIC) strcat(iface, "*");
+		else strcat(iface, "any");
+	}
+	if (print_iface)
+		printf("%s-i %s ", fw->arp.invflags & ARPT_INV_VIA_IN ? "! ": "", iface);
+
+	print_iface = 0;
+	iface[0] = '\0';
+
+	if (fw->arp.outiface[0] != '\0') {
+		strcat(iface, fw->arp.outiface);
+		print_iface = 1;
+	}
+	else if (format & FMT_VIA) {
+		print_iface = 1;
+		if (format & FMT_NUMERIC) strcat(iface, "*");
+		else strcat(iface, "any");
+	}
+	if (print_iface)
+		printf("%s-o %s ", fw->arp.invflags & ARPT_INV_VIA_OUT ? "! " : "", iface);
 
 	if (fw->arp.smsk.s_addr != 0L) {
 		printf("%s", fw->arp.invflags & ARPT_INV_SRCIP
@@ -1370,7 +1371,7 @@ after_devdst:
 			/* Print the target information. */
 			target->print(&fw->arp, t, format & FMT_NUMERIC);
 	} else if (t->u.target_size != sizeof(*t))
-		printf("[%u bytes of unknown target data] ",
+		printf("[%zu bytes of unknown target data] ",
 		       t->u.target_size - sizeof(*t));
 
 	if (!(format & FMT_NOCOUNTS)) {
@@ -2039,7 +2040,7 @@ int do_command(int argc, char *argv[], char **table, arptc_handle_t *handle)
 			check_inverse(optarg, &invert, &optind, argc);
 			set_option(&options, OPT_P_TYPE, &fw.arp.invflags,
 				   invert);
-			if (get16_and_mask(argv[optind - 1], &fw.arp.arpro, &fw.arp.arpro_mask, 10)) {
+			if (get16_and_mask(argv[optind - 1], &fw.arp.arpro, &fw.arp.arpro_mask, 0)) {
 				if (strcasecmp(argv[optind-1], "ipv4"))
 					exit_error(PARAMETER_PROBLEM, "Problem with specified protocol type");
 				fw.arp.arpro = htons(0x800);
@@ -2159,12 +2160,12 @@ int do_command(int argc, char *argv[], char **table, arptc_handle_t *handle)
 					"-%c requires packet and byte counter",
 					opt2char(OPT_COUNTERS));
 
-			if (sscanf(pcnt, "%llu", &fw.counters.pcnt) != 1)
+			if (sscanf(pcnt, "%"PRIu64, &fw.counters.pcnt) != 1)
 				exit_error(PARAMETER_PROBLEM,
 					"-%c packet counter not numeric",
 					opt2char(OPT_COUNTERS));
 
-			if (sscanf(bcnt, "%llu", &fw.counters.bcnt) != 1)
+			if (sscanf(bcnt, "%"PRIu64, &fw.counters.bcnt) != 1)
 				exit_error(PARAMETER_PROBLEM,
 					"-%c byte counter not numeric",
 					opt2char(OPT_COUNTERS));
